unit MabRGSav;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MabRegSave, DBGrids;

type
   TMabDBGridProperty = (gpPosition, gpSize, gpColumnWidths, gpColumnTitles);
   TMabDBGridProperties = set of TMabDBGridProperty;
   TMabRegDBGridSave = class(TMabRegSave)
   private
      { Private declarations }
      FGrid: TDBGrid;
      FRemember: TMabDBGridProperties;
   protected
      { Protected declarations }
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;

      procedure Load;
      procedure Save;
   published
      { Published declarations }
      property Grid: TDBGrid read FGrid write FGrid;
      property Key;
      property Remember: TMabDBGridProperties read FRemember write FRemember;
   end;

implementation

constructor TMabRegDBGridSave.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FRemember := [gpPosition, gpSize, gpColumnWidths];
end;

procedure TMabRegDBGridSave.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = Grid) then
      Grid := nil;
end;

procedure TMabRegDBGridSave.Load;
var
   PName, PMesg: PChar;
   i: Integer;
begin
   try
      if Key = '' then
         raise Exception.Create('Key must be have a value');
      if Grid = nil then
         raise Exception.Create('Grid must be specified');

      with Grid do
      begin
            if FRegistry.OpenKey(Key + Owner.ClassName + '\' + Name + '\Columns', True) then
            begin
               for i := 0 to Columns.Count - 1 do
                  with Columns[i] do
                  begin
                     if gpColumnWidths in Remember then
                        Width := FRegistry.ReadInteger(IntToStr(i), 'Width', Width);
                     if gpColumnTitles in Remember then
                        Title.Caption := FRegistry.ReadString(IntToStr(i), 'Title', Title.Caption);
                  end;

               FRegistry.CloseKey;
            end;

         if gpPosition in Remember then
            if FRegistry.OpenKey(Key + Owner.ClassName, True) then
            begin
               Top := FRegistry.ReadInteger(Name, 'Top', Top);
               Left := FRegistry.ReadInteger(Name, 'Left', Left);

               FRegistry.CloseKey;
            end;

         if gpSize in Remember then
            if FRegistry.OpenKey(Key + Owner.ClassName, True) then
            begin
               Height := FRegistry.ReadInteger(Name, 'Height', Height);
               Width := FRegistry.ReadInteger(Name, 'Width', Width);

               FRegistry.CloseKey;
            end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

procedure TMabRegDBGridSave.Save;
var
   PName, PMesg: PChar;
   i: Integer;
begin
   try
      if Key = '' then
         raise Exception.Create('Key must be have a value');
      if Grid = nil then
         raise Exception.Create('Grid must be specified');

      with Grid do
      begin
         if FRegistry.OpenKey(Key + Owner.ClassName + '\' + Name + '\Columns', True) then
         begin
            for i := 0 to Columns.Count - 1 do
               with Columns[i] do
               begin
                  if gpColumnWidths in Remember then
                     FRegistry.WriteInteger(IntToStr(i), 'Width', Width);
                  if gpColumnTitles in Remember then
                     FRegistry.WriteString(IntToStr(i), 'Title', Title.Caption);
               end;

            FRegistry.CloseKey;
         end;

         if FRegistry.OpenKey(Key + Owner.ClassName, True) then
         begin
            if gpPosition in Remember then
            begin
               FRegistry.WriteInteger(Name, 'Top', Top);
               FRegistry.WriteInteger(Name, 'Left', Left);
            end;

            if gpSize in Remember then
            begin
               FRegistry.WriteInteger(Name, 'Height', Height);
               FRegistry.WriteInteger(Name, 'Width', Width);
            end;
            FRegistry.CloseKey;
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

end.
