unit Mabrup;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Menus;

type
  TMabRollUp = class(TCustomControl)
  private
    { Private declarations }
    SavedHeight: Integer;

    procedure DrawCaption;
  protected
    { Protected declarations }
    FBevelStyle: TPanelBevel;
    FBorderWidth: Byte;
    FCaption: TCaption;
    FCaptionHeight: Byte;
    FMovable: Boolean;
    FRolledUp: Boolean;
    FSizable: Boolean;

    FOnRolling: TNotifyEvent;
    FOnRolled: TNotifyEvent;
    
    procedure SetBevelStyle(ABevelStyle: TPanelBevel);
    procedure SetCaption(ACaption: TCaption);
    procedure SetCaptionHeight(ACaptionHeight: Byte);
    procedure SetBorderWidth(ABorderWidth: Byte);
    procedure SetRolledUp(ARolledUp: Boolean);

    procedure Paint; override;
    procedure WMLButtonUp(var msg: TWMLButtonUp); message WM_LBUTTONUP;
    procedure WMNCHitTest(var msg: TWMNCHitTest); message WM_NCHITTEST;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property BevelStyle: TPanelBevel read FBevelStyle write SetBevelStyle default bvRaised;
    property BorderWidth: Byte read FBorderWidth write SetBorderWidth;
    property Caption: TCaption read FCaption write SetCaption;
    property CaptionHeight: Byte read FCaptionHeight write SetCaptionHeight;
    property Movable: Boolean read FMovable write FMovable default False;
    property RolledUp: Boolean read FRolledUp write SetRolledUp default False;
    property Sizable: Boolean read FSizable write FSizable default False;

    { events }
    property OnRolling: TNotifyEvent read FOnRolling write FOnRolling;
    property OnRolled: TNotifyEvent read FOnRolled write FOnRolled;
    
    { republished properties }
    property Align;
    property Font;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnDragDrop;
    property OnDragOver;
    property PopupMenu;
    property OnMouseMove;
  end;

implementation

{$IFDEF WIN32}
{$R mabrup32.res}
{$ELSE}
{$R mabrup.res}
{$ENDIF}

constructor TMabRollUp.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

	ControlStyle := [csAcceptsControls, csClickEvents, csSetCaption];
   FBevelStyle := bvRaised;
   FBorderWidth := GetSystemMetrics(SM_CYDLGFRAME);
   FCaptionHeight := GetSystemMetrics(SM_CYCAPTION);
   FMovable := False;
   FRolledUp := False;
   FSizable := False;

   FOnRolling := nil;
   FOnRolled := nil;

   SetBounds(Left, Top, 128, 128);
end;

procedure TMabRollUp.SetBevelStyle(ABevelStyle: TPanelBevel);
begin
   if ABevelStyle <> FBevelStyle then
   begin
      FBevelStyle := ABevelStyle;
      Refresh;
   end;
end;

procedure TMabRollUp.SetCaption(ACaption: TCaption);
begin
   if ACaption <> FCaption then
   begin
      FCaption := ACaption;
      DrawCaption;
   end;
end;

procedure TMabRollUp.SetCaptionHeight(ACaptionHeight: Byte);
begin
   if ACaptionHeight <> FCaptionHeight then
   begin
      FCaptionHeight := ACaptionHeight;
      Refresh;
   end;
end;

procedure TMabRollUp.SetBorderWidth(ABorderWidth: Byte);
begin
   if ABorderWidth <> FBorderWidth then
   begin
      FBorderWidth := ABorderWidth;
      Refresh;
   end;
end;

procedure TMabRollUp.SetRolledUp(ARolledUp: Boolean);
var
   i: Integer;
begin
   if ARolledUp <> FRolledUp then
   begin
      if Assigned(FOnRolling) then FOnRolling(Self);

      FRolledUp := ARolledUp;
      for i := 0 to ControlCount - 1 do
         Controls[i].Enabled := not RolledUp;

      if FRolledUp then
      begin
         SavedHeight := Height;
         Height := CaptionHeight + 2 * BorderWidth;
      end
      else
         Height := SavedHeight;

      if Assigned(FOnRolled) then FOnRolled(Self);
   end;
end;

procedure TMabRollUp.DrawCaption;
var
   BtnBmp: TBitmap;
   TempCaption: array[0..255] of Char;
   rc: TRect;
begin
   with Canvas do
   begin
      {
         Fill the entire caption with the inactive caption color,
         and draw an indented frame around it.
      }
      rc := Rect(BorderWidth, BorderWidth,
             Width - BorderWidth, CaptionHeight + BorderWidth);
      Brush.Color := clInactiveCaption;
      FillRect(rc);
      Frame3d(Canvas, rc, clBtnShadow, clBtnHighlight, 1);

      {
         Write the caption text in the active caption font color,
         clipping inside the valid area.
      }
      with rc do
      begin
         Left := Left + 1;
         Top := Top + 1;
         Right := Right - CaptionHeight - 1;
         Bottom := Bottom - 1;
      end;
      Font.Assign(Self.Font);
      {Font.Color := clCaptionText;}
      StrPCopy(TempCaption, Caption);
      DrawText(Handle, TempCaption, -1, rc, DT_LEFT + DT_SINGLELINE + DT_VCENTER);

      {
         Draw the 'roll up' button.
      }
      Brush.Color := clBtnFace;
      rc := Rect(Width - BorderWidth - CaptionHeight + 2, BorderWidth + 2,
            Width - BorderWidth - 2, BorderWidth + CaptionHeight - 2);
      FillRect(rc);
      Frame3d(Canvas, rc, clBtnHighlight, clBtnShadow, 1);

      BtnBmp := TBitmap.Create;
      try
         if RolledUp then
            BtnBmp.Handle := LoadBitmap(HInstance, 'UNROLL')
         else
            BtnBmp.Handle := LoadBitmap(HInstance, 'ROLLUP');

         BrushCopy(rc, BtnBmp, BtnBmp.Canvas.ClipRect,
            ColorToRGB(BtnBmp.Canvas.Pixels[0,0]));
      finally
         BtnBmp.Free;
      end;
   end;
end;

procedure TMabRollUp.Paint;
var
   rc: TRect;
begin
   inherited Paint;

   rc := Rect(0, 0, Width, Height);
   if FBevelStyle = bvRaised then
      Frame3d(Canvas, rc, clBtnHighlight, clBtnShadow, 1)
   else if FBevelStyle = bvLowered then
      Frame3d(Canvas, rc, clBtnShadow, clBtnHighlight, 1);

   DrawCaption;
end;

procedure TMabRollUp.WMLButtonUp(var msg: TWMLButtonUp);
var
   rc: TRect;
   pt: TPoint;
begin
   inherited;

   if Visible and (Msg.Keys = 0) then
   begin
      rc := Rect(Width - BorderWidth - CaptionHeight + 2, BorderWidth + 2,
            Width - BorderWidth - 2, BorderWidth + CaptionHeight - 2);

      pt.x := Msg.XPos;;
      pt.y := Msg.YPos;

      if PtInRect(rc, pt) then
         RolledUp := not RolledUp;
      BringToFront;
   end;
   Msg.Result := 0;
end;

procedure TMabRollUp.WMNCHitTest(var msg: TWMNCHitTest);
var
   rc: TRect;
   pt: TPoint;
begin
   inherited;

   if Visible and not (csDesigning in ComponentState) then
   begin
      pt.x := Msg.XPos;
      pt.y := Msg.YPos;
      pt := ScreenToClient(pt);

      rc := Rect(BorderWidth, BorderWidth, Width - BorderWidth, Height - BorderWidth);
      if PtInRect(rc, pt) then
      begin
         rc.Bottom := CaptionHeight + BorderWidth;
         if PtInRect(rc, pt) then
         begin
            rc := Rect(Width - BorderWidth - CaptionHeight + 2, BorderWidth + 2,
                  Width - BorderWidth - 2, BorderWidth + CaptionHeight - 2);

            if Movable and not PtInRect(rc, pt) then
            {
               RolledUp := not RolledUp
            else}
               Msg.Result := HTCAPTION;
         end;
      end
      else if Sizable then
         with pt do
         begin
            if X < BorderWidth then
            begin
               if Y < CaptionHeight then
                  Msg.Result := HTTOPLEFT
               else if Y > Height - CaptionHeight then
                  Msg.Result := HTBOTTOMLEFT
               else
                  Msg.Result := HTLEFT;
            end
            else if X > Width - BorderWidth then
            begin
               if Y < CaptionHeight then
                  Msg.Result := HTTOPRIGHT
               else if Y > Height - CaptionHeight then
                  Msg.Result := HTBOTTOMRIGHT
               else
                  Msg.Result := HTRIGHT;
            end
            else if Y < BorderWidth then
               Msg.Result := HTTOP
            else if Y > Height - BorderWidth then
               Msg.Result := HTBOTTOM;
         end;
   end;
end;

end.
